# This script converts an OLCI swath from separate netCDF files per-band
# into a single file, user can select bands from command line flags.
# Output file can either be netCDF4 or GeoTIFF

# Command line arguments:
# 1) Input directory
# 2) Output filename
# 3) Optional: Output format, netCDF or GTiff
# 4-) Optional: Band numbers to process (indexed from one) and/or lat/lon to process.

# Comments to: simon.proud@physics.ox.ac.uk
# Use at your own risk, can't see how but it might break things.
# Version 2: User-selectable bands

# Default libraries
import sys
import glob

# User libraries
import numpy as np
import netCDF4
import gdal

import scipy.misc
import scipy

do_lat=False
do_lon=False

bands_to_process=np.zeros((21))
bands_to_process[3]=1
bands_to_process[5]=1
bands_to_process[8]=1
n_bands=3
new_bands=True

# Check command line arguments
lenner	=	len(sys.argv)
if (lenner<3):
	print ('You need to add two arguments: Input directory and output file!')
	quit()
outform='netCDF'
if (lenner==4):
	outform=sys.argv[3]
if (outform !='netCDF' and outform !='PNG' and outform != 'GTiff'):
	print ('Output format must be netCDF, PNG or GTiff!')
	quit()
# Process optional arguments giving band names to process.
if (lenner>4):
	for i in range(4,lenner):
		if (sys.argv[i]=='lat' or sys.argv[i]=='latitude'):
		 	do_lat	=	True
		elif (sys.argv[i]=='Lat' or sys.argv[i]=='Latitude'):
		 	do_lat	=	True
		elif (sys.argv[i]=='lon' or sys.argv[i]=='longitude'):
		 	do_lon	=	True
		elif (sys.argv[i]=='Lon' or sys.argv[i]=='Longitude'):
		 	do_lon	=	True
		else:
			if (new_bands==True):
				bands_to_process[:]=0
				n_bands=0
				new_bands=False
			try:
				band=int(sys.argv[i])-1
			except:
				print ('Cannot convert command line argument:',sys.argv[i],'into a band number!')
				quit()
			if (band<0 or band>20):
				print ('Band numbers must be between 1-21!')
				quit()
			bands_to_process[band]=1
			n_bands+=1

# Set filenames
infile	=	sys.argv[1]
outfile	=	sys.argv[2]

# Find equivalent band names for OLCI
geof		=	glob.glob(infile+'/geo_coordinates.nc')

first		=	True
count		=	0
try:
	bpts	=	(bands_to_process>0).nonzero()[0]
except:
	print ('No bands to process,')
	quit()


# Begin actually doing useful things
print ('Reading input data')

for j in bpts:
	bname=glob.glob(infile+'/Oa'+str(j+1).zfill(2)+'_radiance.nc')
	print (bname)
	if (len(bname)<1):
		print ('Cannot find file for band:',str(j+1).zfill(2))
	if (first==True):
		count		=	0
		first=False
		fid=netCDF4.Dataset(bname[0],'r')
		data=np.array(fid['Oa'+str(j+1).zfill(2)+'_radiance'][:])
		outarr=np.zeros((data.shape[0],data.shape[1],n_bands),dtype=np.float64)
		outarr[:,:,count]=data
		count+=1
	else:
		fid=netCDF4.Dataset(bname[0],'r')
		data=np.array(fid['Oa'+str(j+1).zfill(2)+'_radiance'][:])
		outarr[:,:,count]=data
		count+=1
		
pt=(outarr>65500).nonzero()
outarr[pt]=np.nan

# Get lats and lons if appropriate
if (do_lat or do_lon):
	fid		=	netCDF4.Dataset(geof[0],'r')
	if (do_lat):	lats		=	fid['latitude'][:]
	if (do_lon):	lons		=	fid['longitude'][:]
	fid.close()

# Find dataset shape
shaper	=	outarr.shape

#Save the data, switches netCDF or GTiff as appropriate, netCDf is default. Because: lazy
print ('Saving to output file')

if outform=='netCDF':
	# Make NC file:
	ncfile	=	netCDF4.Dataset(outfile, 'w')
	ncfile.createDimension('x',shaper[0])
	ncfile.createDimension('y',shaper[1])

	#Save variables into file. Lat+Lon are saved for georeferencing (panoply does this by default).
	if (do_lat):
		lato		=	ncfile.createVariable('latitude','f',('x','y'),fill_value=-999)
		lato[:]	=	lats
	if (do_lon):
		lono		=	ncfile.createVariable('longitude','f',('x','y'),fill_value=-999)
		lono[:]	=	lons

	blist=[]
	count=0
	# Save each band, have to do a bit of list shenanegans to get this working.
	for j in bpts:
		blist.append(ncfile.createVariable('Band'+str(j+1).zfill(2),'f',('x','y'),fill_value=65535))
		blist[count][:]=outarr[:,:,count]
		count+=1

if outform=='GTiff':
	# Make TIFF file
	# Note: Bands in file are as follows, assuming we are processing lat/lon:
	# 1) Latitude
	# 2) Longitude
	# 3-) Any optional channels to be processed
	# We can't make a proper geotiff as the lats/lons are not consistent in swath images.
	# This can be fixed by geotransformation/warping using gdalwarp or similar.

	sband=1
	ods=gdal.GetDriverByName('GTiff').Create(outfile,shaper[1],shaper[0],n_bands,gdal.GDT_Float32)
	
	do_lat=False
	do_lon=False

	if (do_lat):
		ods.GetRasterBand(sband).WriteArray(lats)
		ods.GetRasterBand(sband).SetNoDataValue(-999)
		sband+=1
	if (do_lon):
		ods.GetRasterBand(sband).WriteArray(lons)
		ods.GetRasterBand(sband).SetNoDataValue(-999)
		sband+=1
	for i in range(0,n_bands):
		ods.GetRasterBand(sband).WriteArray(outarr[:,:,i])
#		ods.GetRasterBand(sband).SetNoDataValue(65535)
		sband+=1
	del ods
	
if outform=='PNG':
	outimg=outarr[:,:,0:3]
	pt=(np.isfinite(outimg)!=True).nonzero()
	
	maxp=99
	minp=0
	
	rmax=np.nanpercentile(outimg[:,:,0],maxp)
	rmin=np.nanpercentile(outimg[:,:,0],minp)
	gmax=np.nanpercentile(outimg[:,:,1],maxp)
	gmin=np.nanpercentile(outimg[:,:,1],minp)
	bmax=np.nanpercentile(outimg[:,:,2],maxp)
	bmin=np.nanpercentile(outimg[:,:,2],minp)
	
	outimg[pt]=0
	
	r=scipy.misc.bytescale(outimg[:,:,0],cmin=rmin,cmax=rmax)
	g=scipy.misc.bytescale(outimg[:,:,1],cmin=gmin,cmax=gmax)
	b=scipy.misc.bytescale(outimg[:,:,2],cmin=bmin,cmax=bmax)
		
	outarr=np.zeros((outimg.shape[0],outimg.shape[1],3),dtype=np.uint16)
	outarr[:,:,0]=b
	outarr[:,:,1]=g
	outarr[:,:,2]=r
	
	
	scipy.misc.imsave(outfile, outarr)
print ('Done')
